package ch.psi.daq.rest.queries;

import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.List;

import org.apache.commons.lang3.builder.ReflectionToStringBuilder;
import org.springframework.util.Assert;

import ch.psi.daq.cassandra.reader.Ordering;
import ch.psi.daq.hazelcast.query.Aggregation;
import ch.psi.daq.hazelcast.query.AggregationType;
import ch.psi.daq.hazelcast.query.Query;
import ch.psi.daq.hazelcast.query.bin.BinIntervalCalculator;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonSubTypes.Type;
import com.fasterxml.jackson.annotation.JsonTypeInfo;

/**
 * 
 * @author zellweger_c
 *
 */
@JsonTypeInfo(
      use = JsonTypeInfo.Id.NAME,
      include = JsonTypeInfo.As.PROPERTY,
      property = "queryType")
@JsonSubTypes(
{
      @Type(value = PulseRangeQuery.class, name = "pulserange"),
      @Type(value = TimeRangeQuery.class, name = "timerange"),
})
public abstract class AbstractQuery implements Query {

   private List<String> channels;

   private LinkedHashSet<String> fields;

   private List<Aggregation> aggregations;

   private AggregationType aggregationType;

   private boolean aggregateChannels;

   private Ordering ordering;

   /**
    * 
    * @param isOrderDescending whether to add a 'orderBy' clause into the database query
    * @param channelIds all the channelIds (channel names) we want to query
    * @param fields the fields (who map to fields in the DB) we are interested in returning to the
    *        client, needs to be in insertion order (hence the {@link LinkedHashSet} type)
    */
   @JsonCreator
   public AbstractQuery(
         // note that those annotations are needed for the polymorphic
         // mapping to work correctly
         @JsonProperty(value = "ordering") Ordering ordering,
         @JsonProperty(value = "channels") List<String> channelIds,
         @JsonProperty(value = "fields") LinkedHashSet<String> fields,
         @JsonProperty(value = "aggregateChannels") boolean aggregateChannels,
         @JsonProperty(value = "aggregationType") AggregationType aggregationType,
         @JsonProperty(value = "aggregations") List<Aggregation> aggregations) {

      this.ordering = ordering;
      this.aggregateChannels = aggregateChannels;
      this.aggregationType = aggregationType;

      if (channelIds == null || fields == null) {
         throw new IllegalArgumentException("sourceIds and/or fields cannot be null.");
      }
      Assert.notNull(channelIds, "channel name must not be null.");
      Assert.notNull(fields, "field, i.e. property, names must not be null.");

      this.channels = channelIds;
      this.fields = fields;
      this.aggregations = aggregations;
   }
   
   /**
    * {@inheritDoc}
    */
   @Override
   public List<String> getChannels() {
      return Collections.unmodifiableList(channels);
   }

   /**
    * {@inheritDoc}
    */
   @Override
   public Ordering getOrdering() {
      return ordering;
   }
   
   /**
    * {@inheritDoc}
    */
   @Override
   public AggregationType getAggregationType() {
      return aggregationType;
   }
   
   @Override
   public BinIntervalCalculator getBinIntervalCalculator() {
      // TODO
      return null;
   }
   
   public boolean isAggregateChannels() {
      return aggregateChannels;
   }


   public LinkedHashSet<String> getFields() {
      return fields;
   }

   public List<Aggregation> getAggregations() {
      return aggregations;
   }
   
   @Override
   public String toString() {
      return ReflectionToStringBuilder.reflectionToString(this);
   }

}
