package ch.psi.daq.rest.queries;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.List;

import javax.validation.constraints.NotNull;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.psi.daq.cassandra.reader.Ordering;
import ch.psi.daq.hazelcast.query.Aggregation;
import ch.psi.daq.hazelcast.query.AggregationType;
import ch.psi.daq.hazelcast.query.range.QueryRange;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

public class TimeRangeQuery extends AbstractQuery {


   private static final String DATE_FORMAT_STRING = "yyyy/MM/dd hh:mm:ss.SSS";
   private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat(DATE_FORMAT_STRING);
   private static Logger logger = LoggerFactory.getLogger(TimeRangeQuery.class);

   @NotNull
   private long start;
   @NotNull
   private long startNanoOffset;

   @NotNull
   private long end;
   @NotNull
   private long endNanoOffset;


   /**
    * 
    * @param ordering
    * @param channelIds all the sourceIds (channel names) we want to query
    * @param fields the fields (who map to fields in the DB) we are interested in returning to the
    *        client, needs to be in insertion order (hence the {@link LinkedHashSet} type)
    * @param startMillis
    * @param startNanoOffset
    * @param endMillis
    * @param endNanoOffset
    */
   @JsonCreator
   public TimeRangeQuery(
         // note that those annotations are needed for the polymorphic
         // mapping to work correctly
         @JsonProperty(value = "ordering") Ordering ordering,
         @JsonProperty(value = "channels") List<String> channels,
         @JsonProperty(value = "fields") LinkedHashSet<String> fields,
         @JsonProperty(value = "aggregateChannels") boolean aggregateChannels,
         @JsonProperty(value = "aggregationType") AggregationType aggregationType,
         @JsonProperty(value = "aggregations") List<Aggregation> aggregations,
         @JsonProperty(value = "start") long startMillis,
         @JsonProperty(value = "startNanoOffset") long startNanoOffset,
         @JsonProperty(value = "startDateTime") String startDateTime,
         @JsonProperty(value = "end") long endMillis,
         @JsonProperty(value = "endNanoOffset") long endNanoOffset,
         @JsonProperty(value = "endDateTime") String endDateTime) {

      super(ordering, channels, fields, aggregateChannels, aggregationType, aggregations);

      this.start = startMillis;
      this.startNanoOffset = startNanoOffset;
      this.end = endMillis;
      this.endNanoOffset = endNanoOffset;


      if (startDateTime != null && endDateTime != null) {
         logger.info("startDateTime and endDateTime specified. This takes precedence over the start / end fields.");
         try {
            Date startDate = DATE_FORMAT.parse(startDateTime);
            Date endDate = DATE_FORMAT.parse(endDateTime);

            this.start = startDate.getTime();
            this.end = endDate.getTime();
         } catch (ParseException e) {
            logger.error("Parsing the start- and/or endDate was unsuccessful. "
                  + "The format must be '" + DATE_FORMAT_STRING + "'", e);
         }
      }

   }

   public long getStart() {
      return start;
   }

   public void setStart(long start) {
      this.start = start;
   }

   public long getStartNanoOffset() {
      return startNanoOffset;
   }

   public void setStartNanoOffset(long startNanoOffset) {
      this.startNanoOffset = startNanoOffset;
   }

   public long getEnd() {
      return end;
   }

   public void setEnd(long end) {
      this.end = end;
   }

   public long getEndNanoOffset() {
      return endNanoOffset;
   }

   public void setEndNanoOffset(long endNanoOffset) {
      this.endNanoOffset = endNanoOffset;
   }

   /**
    * {@inheritDoc}
    */
   @Override
   public String toString() {
      return ToStringBuilder.reflectionToString(this);
   }

   @Override
   public QueryRange getQueryRange() {
      // TODO Auto-generated method stub
      return null;
   }


}
